@@ -1,6 +1,6 @@
 /*
  * ProFTPD: mod_case -- provides case-insensivity
- * Copyright (c) 2004-2021 TJ Saunders
+ * Copyright (c) 2004-2025 TJ Saunders
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include "conf.h"
 #include "privs.h"
 
-#define MOD_CASE_VERSION	"mod_case/0.9"
+#define MOD_CASE_VERSION	"mod_case/0.9.1"
 
 /* Make sure the version of proftpd is as necessary. */
 #if PROFTPD_VERSION_NUMBER < 0x0001030402
@@ -42,7 +42,7 @@ static int case_expr_eval_cmds(cmd_rec *cmd, array_header *list) {
   int cmd_id, found;
   register unsigned int i;
 
-  for (i = 0; i < list->nelts; i++) { 
+  for (i = 0; i < list->nelts; i++) {
     char *c = ((char **) list->elts)[i];
     found = 0;
 
@@ -172,6 +172,8 @@ static void case_replace_copy_paths(cmd_rec *cmd, const char *proto,
     cmd->arg = pstrcat(cmd->pool, cmd->argv[1], " ", src_path, " ", dst_path,
       NULL);
   }
+
+  pr_cmd_clear_cache(cmd);
 }
 
 static void case_replace_link_paths(cmd_rec *cmd, const char *proto,
@@ -195,6 +197,8 @@ static void case_replace_link_paths(cmd_rec *cmd, const char *proto,
       cmd->argv[1] = cmd->arg;
     }
   }
+
+  pr_cmd_clear_cache(cmd);
 }
 
 static void case_replace_path(cmd_rec *cmd, const char *proto, const char *path,
@@ -213,6 +217,7 @@ static void case_replace_path(cmd_rec *cmd, const char *proto, const char *path,
        */
 
       if (path_index > 0) {
+        unsigned int i;
         char *arg;
 
         arg = pstrdup(cmd->tmp_pool, cmd->arg);
@@ -220,10 +225,23 @@ static void case_replace_path(cmd_rec *cmd, const char *proto, const char *path,
         arg = pstrcat(cmd->pool, arg, path, NULL);
         cmd->arg = arg;
 
+        /* We also need to find the index into cmd->argv to replace.  Look
+         * for the first item that does not start with `-`.
+         */
+        for (i = 1; i < cmd->argc; i++) {
+          if (*((char *) cmd->argv[i]) != '-') {
+            break;
+          }
+        }
+
+        cmd->argv[i] = pstrdup(cmd->pool, path);
+
       } else {
         cmd->arg = pstrdup(cmd->pool, path);
       }
 
+      pr_cmd_clear_cache(cmd);
+
     } else {
       char *arg, *dup_path;
       array_header *argv;
@@ -318,6 +336,7 @@ static void case_replace_path(cmd_rec *cmd, const char *proto, const char *path,
         pr_cmd_strcmp(cmd, "STAT") == 0) {
       cmd->arg = pstrdup(cmd->pool, path);
     }
+    pr_cmd_clear_cache(cmd);
 
     return;
   }
@@ -827,19 +846,20 @@ MODRET case_pre_link(cmd_rec *cmd) {
 
 /* usage: CaseEngine on|off */
 MODRET set_caseengine(cmd_rec *cmd) {
-  int bool;
+  int engine;
   config_rec *c;
 
   CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL);
   CHECK_ARGS(cmd, 1);
 
-  bool = get_boolean(cmd, 1);
-  if (bool == -1)
+  engine = get_boolean(cmd, 1);
+  if (engine == -1) {
     CONF_ERROR(cmd, "expected Boolean parameter");
+  }
 
   c = add_config_param(cmd->argv[0], 1, NULL);
   c->argv[0] = pcalloc(c->pool, sizeof(unsigned int));
-  *((unsigned int *) c->argv[0]) = bool;
+  *((unsigned int *) c->argv[0]) = engine;
 
   return PR_HANDLED(cmd);
 }
